package com.qmth.distributed.print.api.mark;


import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.entity.ExamCard;
import com.qmth.distributed.print.business.entity.ExamTaskDetail;
import com.qmth.distributed.print.business.service.ExamCardService;
import com.qmth.distributed.print.business.service.ExamTaskDetailService;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.bean.vo.PaperInfoVo;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.CardCreateMethodEnum;
import com.qmth.teachcloud.common.enums.log.CustomizedOperationTypeEnum;
import com.qmth.teachcloud.common.util.ExamTaskUtil;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkQuestionDto;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.mark.params.MarkObjectiveQuestionParams;
import com.qmth.teachcloud.mark.params.MarkQuestionParams;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.List;

/**
 * <p>
 * 小题信息表 前端控制器
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Api(tags = "评卷-试卷结构")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_MARK + "/question")
public class MarkQuestionController {

    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private ExamTaskDetailService examTaskDetailService;
    @Resource
    private ExamCardService examCardService;

    /**
     * 评卷结构查询
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result list(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                       @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                       @ApiParam(value = "试卷编号", required = true) @RequestParam String paperType) {
        ExamTaskDetail examTaskDetail = examTaskDetailService.getByExamIdAndCourseCodeAndPaperNumber(examId, null, paperNumber);
        List<PaperInfoVo> paperInfoVoList = ExamTaskUtil.parsePaperAttachmentPath(examTaskDetail.getPaperAttachmentIds(), paperType);
        PaperInfoVo paperInfoVo = paperInfoVoList.get(0);
        boolean canCreate = false;
        if (paperInfoVo.getCardId() != null) {
            ExamCard examCard = examCardService.getById(paperInfoVo.getCardId());
            if (examCard != null) {
                if (CardCreateMethodEnum.UPLOAD.equals(examCard.getCreateMethod())) {
                    canCreate = true;
                }
            }
        }
        MarkQuestionDto markQuestionDto = markQuestionService.pageQuestionsByExamIdAndPaperNumber(examId, paperNumber, paperType, canCreate);
        return ResultUtil.ok(markQuestionDto);
    }

    /**
     * 评卷结构保存
     */
    @ApiOperation(value = "提交")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.ADD)
    public Result save(@RequestBody MarkQuestionParams markQuestionParams) {
        markQuestionService.saveQuestions(markQuestionParams);
        return ResultUtil.ok(true);
    }

    /**
     * 客观题标答查询
     */
    @ApiOperation(value = "客观题标答查询")
    @RequestMapping(value = "/objective/list", method = RequestMethod.POST)
    public Result listObjective(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                                @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) {
        List<MarkQuestion> markQuestionList = markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, null, true);
        return ResultUtil.ok(markQuestionList);
    }

    /**
     * 客观题标答保存
     */
    @ApiOperation(value = "客观题标答提交")
    @RequestMapping(value = "/objective/save", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.ADD)
    public Result saveObjective(@RequestBody MarkObjectiveQuestionParams markObjectiveQuestionParams) {
        markQuestionService.saveObjectiveQuestions(markObjectiveQuestionParams);
        return ResultUtil.ok(true);
    }

    /**
     * 主观题标答文件查询
     */
    @ApiOperation(value = "主观题标答文件查询")
    @RequestMapping(value = "/subjective/get", method = RequestMethod.POST)
    public Result getSubjective(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                                @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) {
        return ResultUtil.ok(markQuestionService.previewAnswerFileByExamIdAndPaperNumber(examId, paperNumber), null);
    }

    /**
     * 主观题标答文件上传
     */
    @ApiOperation(value = "主观题标答文件上传")
    @RequestMapping(value = "/subjective/upload", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.ADD)
    public Result uploadSubjective(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                                   @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                                   @ApiParam(value = "文件", required = true) @RequestParam MultipartFile file,
                                   @ApiParam(value = "文件md5", required = true) @RequestParam String md5) {

        return ResultUtil.ok(markQuestionService.uploadSubjectiveFile(examId, paperNumber, file, md5),"");
    }
}
