package com.qmth.teachcloud.mark.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.qmth.teachcloud.common.enums.QuestionType;
import com.qmth.teachcloud.mark.bean.vo.parseCard.*;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

/**
 * 题卡格式中试卷结构解析
 */
public class CardParseUtils {
    private final static Logger log = LoggerFactory.getLogger(CardParseUtils.class);

    public static List<Struct> parseCardContent(String content) {
        JSONObject jsonObject = JSON.parseObject(content, JSONObject.class);
        List<Pages> pages = JSON.parseArray(jsonObject.getString("pages"), Pages.class);
        List<StructData> structDataList = new ArrayList<>();
        String curTopicId = "0";
        int i = 1;
        for (Pages page : pages) {
            for (Columns column : page.getColumns()) {
                for (Elements topic : column.getElements()) {
                    if (topic.getParent() == null || curTopicId.equals(topic.getParent().getId())) {
                        continue;
                    }
                    curTopicId = topic.getParent().getId();
                    int questionsCount = 1, startNumber = 1;
                    if (topic.getType().equals("COMPOSITION")) {
                        long composition = structDataList.stream().filter(m -> m.getCardTopicType().equals("COMPOSITION") && m.getMainNumber() == topic.getParent().getTopicNo() * 1).count();
                        if (composition > 0) {
                            continue;
                        }
                    } else {
                        questionsCount = topic.getParent().getQuestionsCount();
                        startNumber = topic.getParent().getStartNumber() == null ? 1 : topic.getParent().getStartNumber();
                    }

                    QuestionType questionType = getQuestionType(topic);
                    if (questionType == null) {
                        continue;
                    }
                    StructData structData = new StructData();
                    structData.setType(questionType.getValue());
                    structData.setMainTitle(questionType.getName());
                    structData.setObjective(questionType.getObjective());
                    structData.setqType(questionType.getObjective() ? "objective" : "subjective");
                    structData.setMainNumber(topic.getParent().getTopicNo() * 1);
                    structData.setMainTitle(topic.getParent().getTopicName());
                    structData.setQuestionsCount(questionsCount);
                    structData.setStartNumber(startNumber);
                    structData.setCardTopicType(topic.getType());
                    if ("FILL_QUESTION".equals(topic.getType())) {
                        structData.setOptionCount(topic.getOptionCount());
                    }
                    // 偶数页
                    if (i % 2 == 0) {
                        structData.setPaperIndex(i / 2);
                        structData.setPageIndex(i % 2 + 2);
                    }
                    // 奇数页
                    else {
                        structData.setPaperIndex(i / 2 + 1);
                        structData.setPageIndex(i % 2);
                    }
                    structDataList.add(structData);
                }
            }
            i++;
        }

        List<Struct> structList = new ArrayList<>();
        for (StructData structData : structDataList) {
            int startNumber = structData.getStartNumber();
            for (int i1 = 0; i1 < structData.getQuestionsCount(); i1++) {
                Struct struct = new Struct();
                struct.setqType(structData.getqType());
                struct.setMainNumber(structData.getMainNumber());
                struct.setSubNumber(startNumber + i1);
                struct.setMainTitle(structData.getMainTitle());
                struct.setType(structData.getType());
                struct.setObjective(structData.getObjective());
                struct.setTypeName(structData.getTypeName());
                struct.setOptionCount(structData.getOptionCount());
                struct.setPaperIndex(structData.getPaperIndex());
                struct.setPageIndex(structData.getPageIndex());
                structList.add(struct);
            }
        }

        structList.stream().sorted(Comparator.comparingInt(Struct::getMainNumber).thenComparing(Struct::getSubNumber));
        return structList;
    }

    /**
     * 获取题卡页数
     *
     * @param content 题卡内容
     * @param isPaper 是否返回张数（页数/2）
     */
    public static int calcPaperCount(String content, Boolean isPaper) {
        if (StringUtils.isNotBlank(content)) {
            JSONObject jsonObject = JSON.parseObject(content, JSONObject.class);
            List<Pages> pages = JSON.parseArray(jsonObject.getString("pages"), Pages.class);
            if (CollectionUtils.isNotEmpty(pages)) {
                int pageSize = pages.size();
                return isPaper ? pageSize / 2 : pageSize;
            }
        }
        return 0;

    }

    private static QuestionType getQuestionType(Elements topic) {
        String type = topic.getType();
        if ("COMPOSITION".equals(type) || "EXPLAIN".equals(type)) {
            return QuestionType.findByValue(5);
        }
        if ("FILL_LINE".equals(type)) {
            return QuestionType.findByValue(4);
        }
        if ("FILL_QUESTION".equals(type)) {
            if (topic.getBoolean()) {
                return QuestionType.findByValue(3);
            }
            if (topic.getMultiply()) {
                return QuestionType.findByValue(2);
            }
            return QuestionType.findByValue(1);
        }
        return null;
    }

    public static void main(String[] args) {
        String content = "{\"version\":\"2.0.0\",\"cardType\":\"STANDARD\",\"cardConfig\":{\"id\":\"437532692649615360\",\"createId\":\"437311641944596480\",\"createTime\":1695256722943,\"updateId\":null,\"updateTime\":1695256722943,\"schoolId\":\"1\",\"orgId\":\"437306348707782656\",\"name\":\"测试规则\",\"examNumberStyle\":\"PRINT\",\"examNumberDigit\":10,\"paperType\":\"PRINT\",\"examAbsent\":true,\"writeSign\":true,\"discipline\":true,\"requiredFields\":[{\"code\":\"studentName\",\"name\":\"姓名\",\"enable\":true,\"contentHtml\":\"<i>姓</i> <i>名</i>\"}],\"extendFields\":[{\"code\":\"studentCode\",\"name\":\"学号\",\"enable\":true,\"contentHtml\":\"<i>学</i> <i>号</i>\"},{\"code\":\"courseCode\",\"name\":\"课程代码\",\"enable\":true,\"contentHtml\":\"<i>课</i> <i>程</i> <i>代</i> <i>码</i>\"},{\"code\":\"courseName\",\"name\":\"课程名称\",\"enable\":true,\"contentHtml\":\"<i>课</i> <i>程</i> <i>名</i> <i>称</i>\"},{\"code\":\"paperNumber\",\"name\":\"试卷编号\",\"enable\":false},{\"code\":\"examPlace\",\"name\":\"校区\",\"enable\":false},{\"code\":\"examRoom\",\"name\":\"考试教室\",\"enable\":false},{\"code\":\"examDate\",\"name\":\"考试日期\",\"enable\":false},{\"code\":\"examTime\",\"name\":\"考试时间\",\"enable\":false},{\"code\":\"clazzName\",\"name\":\"班级\",\"enable\":false}],\"titleRule\":\"${schoolName}\",\"attention\":\"无\",\"objectiveAttention\":\"\",\"subjectiveAttention\":\"\",\"enable\":true,\"remark\":\"\",\"fillNumber\":10,\"orgIds\":null,\"pageSize\":\"A3\",\"columnNumber\":2,\"columnGap\":20,\"showForbidArea\":false,\"undertakingEnable\":false,\"undertakingBody\":\"在考试过程中，严格遵守考场纪律，服从监考老师安排，不请他（她）人代考，不将通讯设备、电子设备带入考场，不携带与有考试内容相关的纸质材料，不扰乱考场秩序，如违反，自愿承担相应责任。\",\"cardDesc\":\"\",\"makeMethod\":\"SELF\",\"aOrB\":false,\"cardTitle\":\"测试学校_1\"},\"pages\":[{\"type\":\"PAGE\",\"pageSize\":\"A3\",\"columnNumber\":2,\"columnGap\":20,\"locators\":{\"top\":[{\"id\":\"locator-0-00\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"},{\"id\":\"locator-0-01\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"}],\"bottom\":[{\"id\":\"locator-0-10\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"}]},\"globals\":[],\"columns\":[{\"type\":\"COLUMN\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\",\"isFull\":false,\"elements\":[{\"type\":\"CARD_HEAD\",\"x\":0,\"y\":0,\"w\":703,\"h\":384,\"cardTitle\":\"测试学校_1\",\"cardDesc\":\"\",\"aOrB\":false,\"paperType\":\"PRINT\",\"examAbsent\":true,\"writeSign\":true,\"fillNumber\":10,\"examNumberStyle\":\"PRINT\",\"businessParams\":[],\"attention\":\"无\",\"objectiveAttention\":\"\",\"subjectiveAttention\":\"\",\"columnNumber\":2,\"isSimple\":false,\"sign\":\"head\",\"id\":\"element-66c3fqdki9bfi8n8\",\"createId\":\"437311641944596480\",\"createTime\":1695256722943,\"updateId\":null,\"updateTime\":1695256722943,\"schoolId\":\"1\",\"orgId\":\"437306348707782656\",\"name\":\"测试规则\",\"examNumberDigit\":10,\"discipline\":true,\"requiredFields\":[{\"code\":\"studentName\",\"name\":\"姓名\",\"enable\":true,\"contentHtml\":\"<i>姓</i> <i>名</i>\"}],\"extendFields\":[{\"code\":\"studentCode\",\"name\":\"学号\",\"enable\":true,\"contentHtml\":\"<i>学</i> <i>号</i>\"},{\"code\":\"courseCode\",\"name\":\"课程代码\",\"enable\":true,\"contentHtml\":\"<i>课</i> <i>程</i> <i>代</i> <i>码</i>\"},{\"code\":\"courseName\",\"name\":\"课程名称\",\"enable\":true,\"contentHtml\":\"<i>课</i> <i>程</i> <i>名</i> <i>称</i>\"},{\"code\":\"paperNumber\",\"name\":\"试卷编号\",\"enable\":false},{\"code\":\"examPlace\",\"name\":\"校区\",\"enable\":false},{\"code\":\"examRoom\",\"name\":\"考试教室\",\"enable\":false},{\"code\":\"examDate\",\"name\":\"考试日期\",\"enable\":false},{\"code\":\"examTime\",\"name\":\"考试时间\",\"enable\":false},{\"code\":\"clazzName\",\"name\":\"班级\",\"enable\":false}],\"titleRule\":\"${schoolName}\",\"enable\":true,\"remark\":\"\",\"orgIds\":null,\"pageSize\":\"A3\",\"columnGap\":20,\"showForbidArea\":false,\"undertakingEnable\":false,\"undertakingBody\":\"在考试过程中，严格遵守考场纪律，服从监考老师安排，不请他（她）人代考，不将通讯设备、电子设备带入考场，不携带与有考试内容相关的纸质材料，不扰乱考场秩序，如违反，自愿承担相应责任。\",\"makeMethod\":\"SELF\",\"key\":\"ilnrk6jgdns9dpoo\",\"isCovered\":false},{\"type\":\"TOPIC_HEAD\",\"x\":0,\"y\":0,\"w\":0,\"h\":10,\"content\":\"\",\"typeName\":\"客观题\",\"isColumnFirst\":false,\"sign\":\"objective\",\"id\":\"element-fe7i532a7n5o1na8\",\"key\":\"29spsc0o8sfj8298\"},{\"id\":\"element-6vbvpnu8fgd792ro\",\"topicNo\":1,\"topicName\":\"单选\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":4,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":false,\"booleanType\":\"√,×\",\"isMultiply\":false,\"key\":\"2d8b9r1oa9m5u8ti\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":136,\"minHeight\":136,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":5,\"questionGap\":8,\"groupGap\":27,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\",\"parent\":{\"id\":\"element-0iuf9a9o8snvj2r8\",\"topicNo\":1,\"topicName\":\"单选\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":4,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":false,\"booleanType\":\"√,×\",\"isMultiply\":false,\"key\":\"9ehr2ht8s5ek8gig\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":114,\"minHeight\":114,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":4,\"questionGap\":8,\"groupGap\":30,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\"},\"isLast\":true,\"elementSerialNo\":0},{\"id\":\"element-8ual4ro46gho5upg\",\"topicNo\":2,\"topicName\":\"多选\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":4,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":false,\"booleanType\":\"√,×\",\"isMultiply\":true,\"key\":\"7cq53gpogj4baib8\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":136,\"minHeight\":136,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":5,\"questionGap\":8,\"groupGap\":27,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\",\"parent\":{\"id\":\"element-envusg0b01773rj4\",\"topicNo\":2,\"topicName\":\"多选\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":4,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":false,\"booleanType\":\"√,×\",\"isMultiply\":true,\"key\":\"gum783aghml1oupo\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":114,\"minHeight\":114,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":4,\"questionGap\":8,\"groupGap\":30,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\"},\"isLast\":true,\"elementSerialNo\":1},{\"type\":\"TOPIC_HEAD\",\"x\":0,\"y\":0,\"w\":0,\"h\":10,\"content\":\"\",\"typeName\":\"主观题\",\"isColumnFirst\":false,\"sign\":\"subjective\",\"id\":\"element-49jf099gel7q2jdl\",\"key\":\"a5utv8kg4olgfgag\"},{\"id\":\"element-1jmao1rgtigdrjmg\",\"key\":\"a188ic582j35atfg\",\"type\":\"FILL_LINE\",\"x\":0,\"y\":0,\"w\":703,\"h\":74,\"minHeight\":74,\"sign\":\"subjective\",\"topicName\":\"填空\",\"topicNo\":3,\"startNumber\":1,\"questionsCount\":2,\"questionNumberPerLine\":2,\"lineNumberPerQuestion\":1,\"lineSpacing\":40,\"questionDirection\":\"vertical\",\"questionLineType\":\"norm\",\"questionLineNums\":[{\"no\":1,\"count\":1},{\"no\":2,\"count\":1}],\"numberPre\":\"\",\"isCovered\":false,\"endNumber\":4,\"parent\":{\"id\":\"element-57oee34g1ha1i8np\",\"key\":\"6cobugt8g06ue6bk\",\"type\":\"FILL_LINE\",\"x\":0,\"y\":0,\"w\":703,\"h\":40,\"minHeight\":40,\"sign\":\"subjective\",\"topicName\":\"填空\",\"topicNo\":3,\"startNumber\":1,\"questionsCount\":4,\"questionNumberPerLine\":2,\"lineNumberPerQuestion\":1,\"lineSpacing\":40,\"questionDirection\":\"vertical\",\"questionLineType\":\"norm\",\"questionLineNums\":[],\"numberPre\":\"\",\"isCovered\":false,\"endNumber\":4},\"isLast\":false,\"elementSerialNo\":2},{\"id\":\"element-sagfto7qido3nk2o\",\"key\":\"81gdas586m1ubj7g\",\"type\":\"FILL_LINE\",\"x\":0,\"y\":0,\"w\":703,\"h\":40,\"minHeight\":40,\"sign\":\"subjective\",\"topicName\":\"填空\",\"topicNo\":3,\"startNumber\":3,\"questionsCount\":2,\"questionNumberPerLine\":2,\"lineNumberPerQuestion\":1,\"lineSpacing\":40,\"questionDirection\":\"vertical\",\"questionLineType\":\"norm\",\"questionLineNums\":[{\"no\":3,\"count\":1},{\"no\":4,\"count\":1}],\"numberPre\":\"\",\"isCovered\":false,\"endNumber\":4,\"parent\":{\"id\":\"element-57oee34g1ha1i8np\",\"key\":\"6cobugt8g06ue6bk\",\"type\":\"FILL_LINE\",\"x\":0,\"y\":0,\"w\":703,\"h\":40,\"minHeight\":40,\"sign\":\"subjective\",\"topicName\":\"填空\",\"topicNo\":3,\"startNumber\":1,\"questionsCount\":4,\"questionNumberPerLine\":2,\"lineNumberPerQuestion\":1,\"lineSpacing\":40,\"questionDirection\":\"vertical\",\"questionLineType\":\"norm\",\"questionLineNums\":[],\"numberPre\":\"\",\"isCovered\":false,\"endNumber\":4},\"isLast\":true,\"elementSerialNo\":3}]},{\"type\":\"COLUMN\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\",\"isFull\":false,\"elements\":[{\"type\":\"TOPIC_HEAD\",\"x\":0,\"y\":0,\"w\":0,\"h\":0,\"content\":\"\",\"typeName\":\"主观题\",\"isColumnFirst\":true,\"sign\":\"subjective\",\"id\":\"element-723kt93qo9p4fd0o\",\"key\":\"9fngiojoscv215hg\"},{\"type\":\"EXPLAIN\",\"x\":0,\"y\":0,\"w\":703,\"h\":458,\"minHeight\":60,\"sign\":\"subjective\",\"topicNo\":4,\"isCovered\":false,\"isLast\":true,\"isExtend\":false,\"showTitle\":true,\"serialNumber\":1,\"elements\":[],\"parent\":{\"id\":\"element-iv2oiq6ofacu690g\",\"key\":\"oh7esnl8e9urf2no\",\"type\":\"EXPLAIN\",\"sign\":\"subjective\",\"topicNo\":4,\"topicName\":\"解答\",\"startNumber\":1,\"questionsCount\":2,\"w\":703,\"endNumber\":2},\"id\":\"element-apulpttd670bqph8\",\"key\":\"ldcmtpqghton0l0o\",\"elementSerialNo\":4},{\"type\":\"EXPLAIN\",\"x\":0,\"y\":0,\"w\":703,\"h\":458,\"minHeight\":60,\"sign\":\"subjective\",\"topicNo\":4,\"isCovered\":false,\"isLast\":true,\"isExtend\":false,\"showTitle\":false,\"serialNumber\":2,\"elements\":[],\"parent\":{\"id\":\"element-iv2oiq6ofacu690g\",\"key\":\"oh7esnl8e9urf2no\",\"type\":\"EXPLAIN\",\"sign\":\"subjective\",\"topicNo\":4,\"topicName\":\"解答\",\"startNumber\":1,\"questionsCount\":2,\"w\":703,\"endNumber\":2},\"id\":\"element-9104lvho8bhuaig8\",\"key\":\"jqva67kg5tpfjfug\",\"elementSerialNo\":5}]}],\"exchange\":{\"card_type\":2,\"page_size\":\"A3\",\"page_image\":\"\",\"locator\":{\"top\":[[0.0504413619,0.0267379679,0.0151324086,0.0142602496],[0.8738965952,0.0267379679,0.0151324086,0.0142602496]],\"bottom\":[[0.0504413619,0.9590017825,0.0151324086,0.0142602496]]},\"fill_locator\":[],\"check_area\":{\"black_line\":[],\"white_line\":[]},\"barcode\":[{\"field\":\"examNumber\",\"area\":[0.0504413619,0.1310160428,0.2184741488,0.1176470588]}],\"qrcode\":[],\"ocr_area\":[],\"info_area\":[[0.0504413619,0.0534759358,0.4432534678,0.3422459893]],\"fill_area\":[{\"field\":\"absent\",\"index\":1,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":null,\"sub_number\":null,\"options\":[[0.3394546028,0.3737188057,0.0151324086,0.0106951872]],\"recog_info\":[]}]},{\"field\":\"breach\",\"index\":1,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":null,\"sub_number\":null,\"options\":[[0.4483764187,0.3737188057,0.0151324086,0.0106951872]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":1,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":1,\"sub_number\":1,\"options\":[[0.0687263556,0.4438502674,0.0113493064,0.008912656],[0.0838587642,0.4438502674,0.0113493064,0.008912656],[0.0989911728,0.4438502674,0.0113493064,0.008912656],[0.1141235813,0.4438502674,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":2,\"options\":[[0.0687263556,0.4598930481,0.0113493064,0.008912656],[0.0838587642,0.4598930481,0.0113493064,0.008912656],[0.0989911728,0.4598930481,0.0113493064,0.008912656],[0.1141235813,0.4598930481,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":3,\"options\":[[0.0687263556,0.4759358289,0.0113493064,0.008912656],[0.0838587642,0.4759358289,0.0113493064,0.008912656],[0.0989911728,0.4759358289,0.0113493064,0.008912656],[0.1141235813,0.4759358289,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":4,\"options\":[[0.0687263556,0.4919786096,0.0113493064,0.008912656],[0.0838587642,0.4919786096,0.0113493064,0.008912656],[0.0989911728,0.4919786096,0.0113493064,0.008912656],[0.1141235813,0.4919786096,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":5,\"options\":[[0.0687263556,0.5080213904,0.0113493064,0.008912656],[0.0838587642,0.5080213904,0.0113493064,0.008912656],[0.0989911728,0.5080213904,0.0113493064,0.008912656],[0.1141235813,0.5080213904,0.0113493064,0.008912656]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":2,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":1,\"sub_number\":6,\"options\":[[0.157629256,0.4438502674,0.0113493064,0.008912656],[0.1727616646,0.4438502674,0.0113493064,0.008912656],[0.1878940731,0.4438502674,0.0113493064,0.008912656],[0.2030264817,0.4438502674,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":7,\"options\":[[0.157629256,0.4598930481,0.0113493064,0.008912656],[0.1727616646,0.4598930481,0.0113493064,0.008912656],[0.1878940731,0.4598930481,0.0113493064,0.008912656],[0.2030264817,0.4598930481,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":8,\"options\":[[0.157629256,0.4759358289,0.0113493064,0.008912656],[0.1727616646,0.4759358289,0.0113493064,0.008912656],[0.1878940731,0.4759358289,0.0113493064,0.008912656],[0.2030264817,0.4759358289,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":9,\"options\":[[0.157629256,0.4919786096,0.0113493064,0.008912656],[0.1727616646,0.4919786096,0.0113493064,0.008912656],[0.1878940731,0.4919786096,0.0113493064,0.008912656],[0.2030264817,0.4919786096,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":10,\"options\":[[0.157629256,0.5080213904,0.0113493064,0.008912656],[0.1727616646,0.5080213904,0.0113493064,0.008912656],[0.1878940731,0.5080213904,0.0113493064,0.008912656],[0.2030264817,0.5080213904,0.0113493064,0.008912656]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":3,\"single\":false,\"horizontal\":true,\"items\":[{\"main_number\":2,\"sub_number\":1,\"options\":[[0.0687263556,0.5650623886,0.0113493064,0.008912656],[0.0838587642,0.5650623886,0.0113493064,0.008912656],[0.0989911728,0.5650623886,0.0113493064,0.008912656],[0.1141235813,0.5650623886,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":2,\"options\":[[0.0687263556,0.5811051693,0.0113493064,0.008912656],[0.0838587642,0.5811051693,0.0113493064,0.008912656],[0.0989911728,0.5811051693,0.0113493064,0.008912656],[0.1141235813,0.5811051693,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":3,\"options\":[[0.0687263556,0.5971479501,0.0113493064,0.008912656],[0.0838587642,0.5971479501,0.0113493064,0.008912656],[0.0989911728,0.5971479501,0.0113493064,0.008912656],[0.1141235813,0.5971479501,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":4,\"options\":[[0.0687263556,0.6131907308,0.0113493064,0.008912656],[0.0838587642,0.6131907308,0.0113493064,0.008912656],[0.0989911728,0.6131907308,0.0113493064,0.008912656],[0.1141235813,0.6131907308,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":5,\"options\":[[0.0687263556,0.6292335116,0.0113493064,0.008912656],[0.0838587642,0.6292335116,0.0113493064,0.008912656],[0.0989911728,0.6292335116,0.0113493064,0.008912656],[0.1141235813,0.6292335116,0.0113493064,0.008912656]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":4,\"single\":false,\"horizontal\":true,\"items\":[{\"main_number\":2,\"sub_number\":6,\"options\":[[0.157629256,0.5650623886,0.0113493064,0.008912656],[0.1727616646,0.5650623886,0.0113493064,0.008912656],[0.1878940731,0.5650623886,0.0113493064,0.008912656],[0.2030264817,0.5650623886,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":7,\"options\":[[0.157629256,0.5811051693,0.0113493064,0.008912656],[0.1727616646,0.5811051693,0.0113493064,0.008912656],[0.1878940731,0.5811051693,0.0113493064,0.008912656],[0.2030264817,0.5811051693,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":8,\"options\":[[0.157629256,0.5971479501,0.0113493064,0.008912656],[0.1727616646,0.5971479501,0.0113493064,0.008912656],[0.1878940731,0.5971479501,0.0113493064,0.008912656],[0.2030264817,0.5971479501,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":9,\"options\":[[0.157629256,0.6131907308,0.0113493064,0.008912656],[0.1727616646,0.6131907308,0.0113493064,0.008912656],[0.1878940731,0.6131907308,0.0113493064,0.008912656],[0.2030264817,0.6131907308,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":10,\"options\":[[0.157629256,0.6292335116,0.0113493064,0.008912656],[0.1727616646,0.6292335116,0.0113493064,0.008912656],[0.1878940731,0.6292335116,0.0113493064,0.008912656],[0.2030264817,0.6292335116,0.0113493064,0.008912656]],\"recog_info\":[]}]}],\"answer_area\":[{\"main_number\":3,\"sub_number\":\"1,2\",\"area\":[0.0504413619,0.6559714795,0.4432534678,0.0659536542]},{\"main_number\":3,\"sub_number\":\"3,4\",\"area\":[0.0504413619,0.7219251337,0.4432534678,0.0356506239]},{\"main_number\":4,\"sub_number\":1,\"area\":[0.5063051702,0.0534759358,0.4432534678,0.4081996435]},{\"main_number\":4,\"sub_number\":2,\"area\":[0.5063051702,0.4616755793,0.4432534678,0.4081996435]}],\"extension\":{\"barcode\":[],\"fill_area\":[],\"ocr_area\":[],\"qrcode\":[]}}},{\"type\":\"PAGE\",\"pageSize\":\"A3\",\"columnNumber\":2,\"columnGap\":20,\"locators\":{\"top\":[{\"id\":\"locator-1-00\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"},{\"id\":\"locator-1-01\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"}],\"bottom\":[{\"id\":\"locator-1-10\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"}]},\"globals\":[],\"columns\":[{\"type\":\"COLUMN\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\",\"isFull\":false,\"elements\":[]},{\"type\":\"COLUMN\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\",\"isFull\":false,\"elements\":[]}],\"exchange\":{\"card_type\":2,\"page_size\":\"A3\",\"page_image\":\"\",\"locator\":{\"top\":[[0.1109709962,0.0267379679,0.0151324086,0.0142602496],[0.9344262295,0.0267379679,0.0151324086,0.0142602496]],\"bottom\":[[0.9344262295,0.9590017825,0.0151324086,0.0142602496]]},\"fill_locator\":[],\"check_area\":{\"black_line\":[],\"white_line\":[]},\"barcode\":[],\"qrcode\":[],\"ocr_area\":[],\"info_area\":[],\"fill_area\":[],\"answer_area\":[],\"extension\":{\"barcode\":[],\"fill_area\":[],\"ocr_area\":[],\"qrcode\":[]}}}]}";
        List<Struct> structList = CardParseUtils.parseCardContent(content);
        System.out.println(JSON.toJSONString(structList));
    }

}
